#clear the memory
rm(list=ls())

#load the dataset
library(xlsx)
vote.data <- read.xlsx(file="vote_catvarclus.xls",header=T,sheetIndex=1)
summary(vote.data)

#active variables of the analysis
vote.active <- subset(vote.data,select=2:7)

#function for calculating the Cramer's v
cramer <- function(y,x){
  K <- nlevels(y)
  L <- nlevels(x)
  n <- length(y)
  chi2 <- chisq.test(y,x,correct=F)
  print(chi2$statistic)
  v <- sqrt(chi2$statistic/(n*min(K-1,L-1)))
  return(v)
}

#similarity matrix
sim <- matrix(1,nrow=ncol(vote.active),ncol=ncol(vote.active))
rownames(sim) <- colnames(vote.active)
colnames(sim) <- colnames(vote.active)
for (i in 1:(nrow(sim)-1)){
  for (j in (i+1):ncol(sim)){
    y <- vote.active[,i]
    x <- vote.active[,j]
    sim[i,j] <- cramer(y,x)
    sim[j,i] <- sim[i,j]
  }
}
print(sim)

#distance matrix
dissim <- as.dist(1-sim)

#clustering
tree <- hclust(dissim,method="ward.D")
plot(tree)
     
#2 subgroups
groups <- cutree(tree,k=2)
print(groups)

#Cramer's v affiliation vs. attributes
cv <- sapply(vote.active,cramer,x=vote.data$affiliation)
print(cv)

#mean of v for each group
m <- tapply(X=cv,INDEX=groups,FUN=mean)
print(m)

#*********************************
# clustering categorical variables
# using ClustOfVar
#*********************************
library(ClustOfVar)
arbre <- hclustvar(X.quali=vote.active)
plot(arbre)

mgroups <- kmeansvar(X.quali=vote.active,init=2,nstart=10)
print(summary(mgroups))

#***************************************************
#cluster analysis on levels of qualitative variables
#***************************************************

#dummy coding
library(ade4)
disj <- acm.disjonctif(vote.active)
print(head(vote.active))
print(head(disj))

#Dice index
dice <- function(m1,m2){
  return(0.5*sum((m1-m2)^2))
}

#Dice index matrix
d2 <- matrix(0,ncol(disj),ncol(disj))
for (j in 1:ncol(disj)){
  for (jprim in 1:ncol(disj)){
    d2[j,jprim] <- dice(disj[,j],disj[,jprim])
  }
}

colnames(d2) <- colnames(disj)
rownames(d2) <- colnames(disj)

#transform the matrix in a dist class
d <- as.dist(sqrt(d2))
print(d)

#cluster analysis on indicator variables
arbre.moda <- hclust(d,method="ward.D2")
plot(arbre.moda)

#create 3 groups
dgroups <- cutree(arbre.moda,k=3)
print(dgroups)

#illustrative variable
illus <- acm.disjonctif(as.data.frame(vote.data$affiliation))
colnames(illus) <- c("democrat","republican")

#distance to illustrative
dice.democrat <- sapply(disj,dice,m2=illus$democrat)
tapply(dice.democrat,dgroups,mean)

dice.republican <- sapply(disj,dice,m2=illus$republican)
tapply(dice.republican,dgroups,mean)

#******************************
#Varclus from the Hmisc package
#******************************
library(Hmisc)
v <- varclus(as.matrix(disj),type="data.matrix",similarity="bothpos",method="ward.D")
plot(v)

#the used matrix of similarity
round(v$sim,2)

#the call to the hclust() procedure
v$hclust$height

#**********************************************
#How to obtain the same results as in varclus()
#**********************************************

#matrx for the bothpos measure
#cf. varclus documentation (Hmisc)
bp <- matrix(0,ncol(disj),ncol(disj))
for (j in 1:ncol(disj)){
  for (jprim in 1:ncol(disj)){
    bp[j,jprim] <- mean(disj[,j]*disj[,jprim])
  }
}

colnames(bp) <- colnames(disj)
rownames(bp) <- colnames(disj)

#dissimliraty measure from bp
dp <- as.dist(1-bp)

#cluster analysis from the dp dissimilarity matrix
arbre.bp <- hclust(dp,method="ward.D")
plot(arbre.bp)

arbre.bp$height

#********************************************************
#cluster analysis from the coordinates of levels provided
#by the MCA multiple correspondence analysis
#********************************************************

#MCA with the ade4 package
acm <- dudi.coa(disj,scannf=F,nf=2)
acm.coord <- data.frame(acm$co)
rownames(acm.coord) <- colnames(disj)

#graphical representation
plot(acm.coord[,1],acm.coord[,2],type="n",xlim=c(-1.25,1.25),ylim=c(-0.5,4.5),xlab="F1",ylab="F2")
text(acm.coord[,1],acm.coord[,2],labels=rownames(acm.coord),cex=0.75,col="blue")
abline(h=0,v=0)

#distance matrix of the levels of qualitative variables
m.acm <- dist(acm.coord,method="euclidian")

#cluster analysis from the distance matrix m.acm
arbre.acm <- hclust(m.acm,method="ward.D")
plot(arbre.acm)
